/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/8
 */

package agentspace.context;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import agentspace.IllegalAgentStateException;
import agentspace.InvalidURLException;
import agentspace.network.SocketClient;
import agentspace.runtime.AgentContainer;
import agentspace.runtime.AgentRuntimeManager;
import agentspace.runtime.event.AgentRuntimeListener;

public class ContextManager {
	AgentRuntimeManager manager;
	Object target;
	
	public ContextManager(AgentRuntimeManager manager) {
		this.manager = manager;
	}
	
	public AgentContainer load(byte[] data) {
		return load(data, true);
	}

	public AgentContainer load(byte[] data, boolean sync) {
		class LoadHandler implements Runnable {
			byte[] data;
			AgentContainer ac;
			public LoadHandler(byte[] data) {
				this.data =data;
			}
			public void run() {
				ac = manager.load(data);
			}
			public AgentContainer getContainer() {
				return ac;
			}
		}
		LoadHandler handler = new LoadHandler(data);
		if (sync) {
			handler.run();
			return handler.getContainer();
		}
		else {
			manager.getThreadPool().startThread(handler);
			return null;
		}
	}

	public void halt(AgentContainer ac) {
		class HaltHandler implements Runnable {
			AgentContainer ac;
			public HaltHandler(AgentContainer ac){
				this.ac = ac;
			}
			public void run() {
				manager.halt(ac);
			}
		}
		HaltHandler handler = new HaltHandler(ac);
		manager.getThreadPool().startThread(handler);
	}

	public void terminate(AgentContainer ac) {
		terminate(ac, false);
	}
	
	public void terminate(AgentContainer ac, boolean sync) { 
		class TerminateHandler implements Runnable {
			AgentContainer ac;
			public TerminateHandler(AgentContainer ac){
				this.ac = ac;
			}
			public void run() {
				manager.terminate(ac);
			}
		}
		TerminateHandler handler = new TerminateHandler(ac);
		if (sync) {
			handler.run();
		}
		else {
			manager.getThreadPool().startThread(handler);
		}
	}

	public AgentContainer duplicate(AgentContainer ac) {
		return duplicate(ac, true);
	}

	public AgentContainer duplicate(AgentContainer ac, boolean sync) {
		class DuplicateHandler implements Runnable {
			AgentContainer ac;
			public DuplicateHandler(AgentContainer ac) {
				this.ac = ac;
			}
			public void run() {
				manager.duplicate(ac);
			}
			public AgentContainer getContainer() {
				return ac;
			}
		}
		DuplicateHandler handler = new DuplicateHandler(ac);
		if (sync) {
			handler.run();
			return handler.getContainer();
		}
		else {
			manager.getThreadPool().startThread(handler);
			return null;
		}
	}

	public byte[] store(AgentContainer ac) 
	throws IllegalAgentStateException, IOException {
		return store(ac, true);
	}

	public byte[] store(AgentContainer ac, boolean sync) {
		byte[] data = null;
		class StoreHandler implements Runnable {
			AgentContainer ac;
			byte[] data;
			public StoreHandler(AgentContainer ac) {
				this.ac = ac;
			}
			public void run() {
				try {
					data = manager.store(ac);					
				}
				catch (Exception ex) {
					ex.printStackTrace();
				}
			}
			public byte[] getData() {
				return data;
			}
		}
		StoreHandler handler = new StoreHandler(ac);
		if (sync) {
			handler.run();
			data = handler.getData();
		}
		else {
			manager.getThreadPool().startThread(handler);
		}
		return data;
	}
	
	public void store(AgentContainer ac, URL url, boolean valid) 
	throws IOException, InvalidURLException, IllegalAgentStateException {
		String filename = url.getFile();
		File file = null;
		if (valid) {
			try {
				file = new File(url.getFile());
			}
			catch (NullPointerException ex) {
				throw new InvalidURLException(url);			
			}
			catch (IllegalArgumentException ex) {
				throw new InvalidURLException(url);			
			}
		}			
		if (file != null) {
			if (!file.exists() || file.canWrite()) {
				byte[] data = store(ac);			
				java.io.FileOutputStream fos = new java.io.FileOutputStream(file);
				fos.write(data, 0, data.length);
				fos.close();
			}
			else {
				throw new IOException("Cannot write file:"+filename);
			}
		}
	}

	public void move(AgentContainer ac, URL url, boolean valid, boolean sync) 
	throws IOException, InvalidURLException, IllegalAgentStateException {
		class MoveHandler implements Runnable {
			AgentContainer ac;
			URL url;
			Exception exception;
			public MoveHandler(AgentContainer ac, URL url) {
				this.ac = ac;
				this.url = url;
			}
			public void move() throws IOException, IllegalAgentStateException, Exception {
				manager.move(ac, url);
			}
			public void run() {
				try {
					move();
				}
				catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
		if (valid) {
			if (!isValidURL(url)) {
				throw new InvalidURLException(url);
			}
		}
		MoveHandler handler = new MoveHandler(ac, url);
		if (!sync) {
			manager.getThreadPool().startThread(handler);
		}
		else {
			try {
				handler.move();
			}
			catch (IOException ex) {
				throw ex;
			}
			catch (IllegalAgentStateException ex) {
				throw ex;
			}
			catch (Exception ex) {
				ex.printStackTrace();
			}
		}
  	}			
	
	
	public boolean isValidURL(URL url) {
		try {
			if (url.getProtocol().equalsIgnoreCase("MATP")) {
				SocketClient.isValidURL(url);
				return true;
			}
			if (url.getProtocol().equalsIgnoreCase("FILE")) {
				File file = new File(url.getFile());
				if (!file.isFile()) {
					return false;
				}
				return true;
			}
		}
		catch (Exception ex){}
		return false;
	}
	
	public synchronized void addAgentRuntimeListener(AgentRuntimeListener listener) {
		manager.addAgentRuntimeListener(listener);
	}

	public synchronized void removeAgentRuntimeListener(AgentRuntimeListener listener) {
		manager.removeAgentRuntimeListener(listener);
	}
}
